/****************************************************************************
  FileName     [ cirFraig.cpp ]
  PackageName  [ cir ]
  Synopsis     [ Define cir FRAIG functions ]
  Author       [ Chung-Yang (Ric) Huang ]
  Copyright    [ Copyleft(c) 2010-2012 LaDs(III), GIEE, NTU, Taiwan ]
****************************************************************************/

#include <cassert>
#include "cirMgr.h"
#include "cirGate.h"
#include "sat.h"
#include "myHash.h"
#include "util.h"

using namespace std;

/*******************************/
/*   Global variable and enum  */
/*******************************/

/**************************************/
/*   Static varaibles and functions   */
/**************************************/

/*******************************************/
/*   Public member functions about fraig   */
/*******************************************/
void
CirMgr::strash()
{
	Hash<HashKey, CirGate*>* myHash = new Hash<HashKey, CirGate*>(getHashSize(_netList.size()/2));
	CirGate* temp;
	HashKey key;
	bool inv = 0;
	for(size_t i=0; i<_netList.size(); ++i)
	{
		if(_netList[i]->getTypeStr() == "AIG")
		{
			key = HashKey((And*)_netList[i]);
			if(myHash->check(key, temp))
			{
				_netList[i]->optgate(temp, inv);
				cout<<"Strashing: "<<temp->getID()
				    <<" merging "<<_netList[i]->getID()<<"..."<<endl;
				_inorder[_netList[i]->getID()] = 0;
				delete _netList[i];
				--A;
			}
			else
				myHash->forceInsert(key, _netList[i]);
		}
	}
	DFSList();
	makeFList();
}

void
CirMgr::fraig()
{
	SatSolver s;
	s.initialize();
	GateList Gates;
	Gates.push_back(_inorder[0]);
	for(size_t i=0; i<_netList.size(); ++i)
	{
		if(_netList[i] != 0)
			if(_netList[i]->getTypeStr()!="PO" && _netList[i]->getTypeStr()!="UNDEF"
				&& _netList[i]->getTypeStr()!="CONST")
				Gates.push_back(_netList[i]);
	}

	initSatGate(s, Gates);
	genProofModel(s, Gates);

	for(size_t i=0; i<_fecList.size(); ++i)
	{//cout<<"size = "<<_fecList.size()<<endl;cout<<"i = "<<i<<endl;
		for(size_t p=0; p<_fecList[i]->size(); ++p)
		{
			for(size_t k=p+1; k<_fecList[i]->size(); )
			{
				if(mergefec((*_fecList[i])[p], (*_fecList[i])[k], s))
				{
					removeData((*_fecList[i]), (*_fecList[i])[k]);
					//--k;
				}
				else ++k;
			}
		}
		delete _fecList[i];
	}

	clearList(_fecList);
	DFSList();
	for(size_t i=0; i<_netList.size(); ++i)
		_netList[i]->setgrpptr(0);
	makeFList();
	//strash();
}

/********************************************/
/*   Private member functions about fraig   */
/********************************************/

void
CirMgr::initSatGate(SatSolver& s, GateList& gatelist)
{
	Var var;
	for(size_t i=0; i<gatelist.size(); ++i)
	{
		var = s.newVar();
		gatelist[i]->setvar(var);
	}
}

void
CirMgr::genProofModel(SatSolver& s, GateList& gatelist)
{
	int id1, id2;
	bool inv1, inv2;
	s.addAigCNF(gatelist[0]->getvar(), gatelist[0]->getvar(), true, gatelist[0]->getvar(), false);
	for(size_t i=0; i<gatelist.size(); ++i)
	{
		if(gatelist[i]->getTypeStr() == "AIG")
		{
			if(!gatelist[i]->isfinfl())
			{
				inv1 = !gatelist[i]->getfinID(id1);
				inv2 = !gatelist[i]->getfinID2(id2);
				s.addAigCNF(gatelist[i]->getvar(), _inorder[id1]->getvar(), inv1, _inorder[id2]->getvar(), inv2);
			}
		}
	}
}

bool
CirMgr::mergefec(CirGate*& gate_a, CirGate*& gate_b, SatSolver& s)
{//cout<<"merge"<<endl;
	bool inv = false;
	if(gate_a->getsim() == gate_b->getsim())
	{
		if(checkfec(gate_a, gate_b, s))
		{
			gate_b->optgate(gate_a, inv);
			_inorder[gate_b->getID()] = 0;
			//removeData(*(gate_b->getgrpptr()), gate_b);
			cout<<"Fraig: "<<gate_a->getID()<<" merging "<<gate_b->getID()<<"..."<<endl;
			delete gate_b;
			--A;
			return true;
		}
	}
	else
	{
		if(checkifec(gate_a, gate_b, s))
		{
			inv = true;
			gate_b->optgate(gate_a, inv);
			_inorder[gate_b->getID()] = 0;
			//removeData(*(gate_b->getgrpptr()), gate_b);
			cout<<"Fraig: "<<gate_a->getID()<<" merging !"<<gate_b->getID()<<"..."<<endl;
			delete gate_b;
			--A;
			return true;
		}
	}
	return false;
}

bool
CirMgr::checkfec(CirGate*& gate_a, CirGate*& gate_b, SatSolver& s)
{//cout<<gate_a->getID()<<" "<<gate_b->getID()<<endl;
	Var a = s.newVar();
	s.assumeRelease();
	s.addXorCNF(a, gate_a->getvar(), false, gate_b->getvar(), false);
	s.assumeProperty(a, true);
	return !(s.assumpSolve());
}

bool
CirMgr::checkifec(CirGate*& gate_a, CirGate*& gate_b, SatSolver& s)
{//cout<<gate_a->getID()<<" !"<<gate_b->getID()<<endl;
	Var a = s.newVar();
	s.assumeRelease();
	s.addXorCNF(a, gate_a->getvar(), false, gate_b->getvar(), false);
	s.assumeProperty(a, false);
	return !(s.assumpSolve());
}

