/****************************************************************************
  FileName     [ cirMgr.h ]
  PackageName  [ cir ]
  Synopsis     [ Define circuit manager ]
  Author       [ Chung-Yang (Ric) Huang ]
  Copyright    [ Copyleft(c) 2008-2012 LaDs(III), GIEE, NTU, Taiwan ]
****************************************************************************/

#ifndef CIR_MGR_H
#define CIR_MGR_H

#include <vector>
#include <string>
#include <fstream>
#include <iostream>

using namespace std;

#include "cirDef.h"
#include "cirGate.h"
#include "sat.h"

extern CirMgr *cirMgr;

// TODO: Define your own data members and member functions
class CirMgr
{
	typedef vector<CirGate*> GateList;
public:
   CirMgr(){}
   ~CirMgr()
	{
		for(size_t i=0; i<_inorder.size(); ++i)
			if(_inorder[i] != 0)
				delete _inorder[i];
		for(size_t i=0; i<_fecList.size(); ++i)
			delete _fecList[i];
	}

   // Access functions
   // return '0' if "gid" corresponds to an undefined gate.
   CirGate* getGate(unsigned gid) const
	{
		if(gid > _inorder.size()-1) return false;
		else return _inorder[gid];
	}

   // Member functions about circuit construction
   bool readCircuit(const string&);

    // Member functions about circuit optimization
	void sweep();
	void optimize();

	// Member functions about simulation
	void randomSim();
	void fileSim(ifstream&);
	void setSimLog(ofstream *logFile) { _simLog = logFile; }

	// Member functions about fraig
	void strash();
	void printFEC() const;
	void fraig();

   // Member functions about circuit reporting
   void printSummary() const;
   void printNetlist() const;
   void printPIs() const;
   void printPOs() const;
   void printFloatGates() const;
	void printFECPairs() const;
   void writeAag(ostream&) const;
	//bool checkFECinv(CirGate*, CirGate*) const;

private:
	int M;
	int I;
	int L;
	int O;
	int A;
	vector<CirGate*>  _piList;
	vector<CirGate*>  _poList;
	vector<CirGate*>  _floating;
	vector<CirGate*>  _netList;
	vector<CirGate*>  _inorder;
	vector<GateList*> _fecList;
	ofstream			   *_simLog;

	CirGate* optcheck(CirGate*&, bool&);
	bool checkfList(CirGate*) const;
	void makeFList();
	void DFSList();
	void simaig();
	int  failtimes();
	void makefirstgroup();
	bool makeFECgroup();
	void cleanFECList();
	//void sortFEC();

	void initSatGate(SatSolver&, GateList&);
	void genProofModel(SatSolver&, GateList&);
	bool mergefec(CirGate*&, CirGate*&, SatSolver&);
	bool checkfec(CirGate*&, CirGate*&, SatSolver&);
	bool checkifec(CirGate*&, CirGate*&, SatSolver&);

	int Char2Bit(char&);
	/*size_t getHSize(size_t s)
	{
		if (s < 8) return 7;
	   if (s < 16) return 13;
	   if (s < 32) return 31;
	   if (s < 64) return 61;
	   if (s < 128) return 127;
	   if (s < 512) return 509;
	   if (s < 2048) return 1499;
	   if (s < 8192) return 4999;
	   if (s < 32768) return 13999;
	   if (s < 131073) return 59999;
	   return 71993;
	}*/

};

#endif // CIR_MGR_H
