/****************************************************************************
  FileName     [ cirSim.cpp ]
  PackageName  [ cir ]
  Synopsis     [ Define cir optimization functions ]
  Author       [ Chung-Yang (Ric) Huang ]
  Copyright    [ Copyleft(c) 2008-2012 LaDs(III), GIEE, NTU, Taiwan ]
****************************************************************************/

#include <cassert>
#include "cirMgr.h"
#include "cirGate.h"
#include "util.h"

using namespace std;

/*******************************/
/*   Global variable and enum  */
/*******************************/

/**************************************/
/*   Static varaibles and functions   */
/**************************************/

/**************************************************/
/*   Public member functions about optimization   */
/**************************************************/
// Remove unused gates
void
CirMgr::sweep()
{
	CirGate::setGlobal();
	for(size_t i=0; i<_netList.size(); ++i)
		_netList[i]->setmark();
	for(size_t i=1; i<_inorder.size(); ++i)
	{
		if(_inorder[i] != 0 && !_inorder[i]->ismarked())
		{
			if(_inorder[i]->getTypeStr() == "AIG")
			{
				_inorder[i]->sweepaig();
				cout<<"Sweeping: AIG("<<_inorder[i]->getID()<<") removed..."<<endl;
				delete _inorder[i];
				_inorder[i] = 0;
				--A;
			}
			else if(_inorder[i]->getTypeStr() == "UNDEF")
			{
				_inorder[i]->sweepaig();
				cout<<"Sweeping: UNDEF("<<_inorder[i]->getID()<<") removed..."<<endl;
				delete _inorder[i];
				_inorder[i] = 0;
			}
		}
	}
	makeFList();
}

// Recursively simplifying from POs;
// _dfsList needs to be reconstructed afterwards
void
CirMgr::optimize()
{
	bool inv = 0;
	CirGate* temp;
	for(size_t i=0; i<_netList.size(); ++i)
	{
		temp = optcheck(_netList[i], inv);
		if(temp)
		{
			_netList[i]->optgate(temp, inv);
			cout<<"Simplifying: "<<temp->getID()<<" merging ";
			if(inv) cout<<"!";
			cout<<_netList[i]->getID()<<"..."<<endl;
			_inorder[_netList[i]->getID()] = 0;
			delete _netList[i];
			--A;
		}
	}
	DFSList();
}

/***************************************************/
/*   Private member functions about optimization   */
/***************************************************/
CirGate*
CirMgr::optcheck(CirGate*& optgate, bool& inv)
{
	if(optgate->getTypeStr() == "AIG")
	{
		int id1, id2;
		if(optgate->getfinID(id1))
		{
			if(id1 == 0) { inv = 0;	return _inorder[0]; }
			else if(optgate->getfinID2(id2))
			{
				if(id2 == 0) { inv = 0; return _inorder[0]; }
				if(id2 == id1) { inv = 0; return _inorder[id1]; }
			}
			else
			{
				if(id2 == 0) { inv = 0; return _inorder[id1]; }
				if(id2 == id1) { inv = 0; return _inorder[0]; }
			}
		}
		else
		{
			if(id1 == 0)
			{
				inv = !optgate->getfinID2(id2);
				return _inorder[id2];
			}
			else if(optgate->getfinID2(id2))
			{
				if(id2 == 0) { inv = 0; return _inorder[0]; }
				if(id2 == id1) { inv = 0; return _inorder[0]; }
			}
			else
			{
				if(id2 == 0) { inv = 1; return _inorder[id1]; }
				if(id2 == id1) { inv = 1; return _inorder[id1]; }
			}
		}
		return 0;
	}
	else return 0;
}
