/****************************************************************************
  FileName     [ cirSim.cpp ]
  PackageName  [ cir ]
  Synopsis     [ Define cir simulation functions ]
  Author       [ Chung-Yang (Ric) Huang ]
  Copyright    [ Copyleft(c) 2008-2012 LaDs(III), GIEE, NTU, Taiwan ]
****************************************************************************/

#include <fstream>
#include <iostream>
#include <iomanip>
#include <cassert>
#include <algorithm>
#include <cmath>
#include "cirMgr.h"
#include "cirGate.h"
#include "util.h"
#include "myHash.h"

using namespace std;

/*******************************/
/*   Global variable and enum  */
/*******************************/

/**************************************/
/*   Static varaibles and functions   */
/**************************************/

/************************************************/
/*   Public member functions about Simulation   */
/************************************************/
void
CirMgr::randomSim()
{
	unsigned count = 0;
	int fail = 0;
	cout<<"MAX_FAILS = "<<failtimes()<<endl;
	if(_fecList.size() == 0) makefirstgroup();
	while(fail < failtimes())
	{
		for(size_t i=0; i<_piList.size(); ++i)
			_piList[i]->simulate((((unsigned)rnGen(INT_MAX))+rnGen(2))<<1|rnGen(2));
		simaig();
		if(!makeFECgroup()) ++fail;
		count += 32;
	}
	//cleanFECList();
	if(count != fail*32)
	{
		for(size_t i=0; i<_netList.size(); ++i)
			_netList[i]->setgrpptr(0);
		for(size_t j=0; j<_fecList.size(); ++j)
			for(size_t k=0; k<_fecList[j]->size(); ++k)
				(*_fecList[j])[k]->setgrpptr((_fecList[j]));
	}
	//sortFEC();
	cout<<count<<" patterns simulated."<<endl;

	//Hash* simHash = new Hash<SimHashKey, vector<CirGate*>* >;
}

void
CirMgr::fileSim(ifstream& patternFile)
{
	vector<string> input;
	string temp1, temp2;
	size_t t;
	int ptime = 0;
	int bcnt = 1;
	bool b = false;
	unsigned ivalue = 0;
	while(!patternFile.eof())
	{
		getline(patternFile, temp1);
		t = temp1.find_first_of(' ');
		while(bcnt)
		{
			if(t == string::npos)
			{
				--bcnt;
				temp2 = temp1;
			}
			else
				temp2 = temp1.substr(0, t);
			//check error1
			if(temp2.size()!=_piList.size() && temp2.size())
			{
				cout<<endl;
				cout<<"Error: Pattern("<<temp2<<") length("<<temp2.size()
				    <<") does not match the number of inputs("<<_piList.size()
					 <<") in a circuit!!"<<endl;
				cout<<ptime<<" patterns simulated."<<endl;
				b = true;
			}
			if(b) break;
			//check error2
			for(size_t i=0; i<temp2.size(); ++i)
			{
				if(temp2[i]!='0' && temp2[i]!='1')
				{
					cout<<endl;
					cout<<"Error: Pattern("<<temp2<<") contains a non-0/1 character('"
					    <<temp2[i]<<"')."<<endl;
					cout<<ptime<<" patterns simulated."<<endl;
					b = true;
					break;
				}
			}
			if(b) break;
			//
			if(temp2.size() != 0) input.push_back(temp2);
			if(bcnt)
			{
				temp1 = temp1.substr(t+1);
				t = temp1.find_first_of(' ');
			}
			//
			if(input.size()%32==0 && input.size()>0)
			{
				ptime += 32;
				if(_fecList.size() == 0) makefirstgroup();
				for(size_t p=0; p<_piList.size(); ++p)
				{
					ivalue = 0;
					for(int k=32; k>0; --k)
					{//cout<<"qqq"<<ivalue<<endl;
						ivalue = ivalue<<1;
						ivalue += Char2Bit(input[k-1][p]);
						//cout<<input[32-k]<<endl;cout<<input[32-k][p]<<endl;
					}
					_piList[p]->simulate(ivalue);//cout<<ivalue<<endl;
				}
				simaig();
				makeFECgroup();
				input.clear();
			}
		}
		bcnt = 1;
		if(b) break;
	}
	if(!b && input.size()>0)
	{//cout<<"check error done"<<endl;
		//if(input.size()%32 == 0) count = input.size()%32;
		//else count = input.size()/32 + 1;
		if(_fecList.size() == 0) makefirstgroup();
		for(size_t i=0; i<input.size(); ++i)
		{//cout<<"i = "<<i<<endl;
			for(size_t p=0; p<_piList.size(); ++p)
			{//cout<<"p = "<<p<<endl;
				ivalue = 0;
				for(int k=input.size(); k>0; --k)
				{//cout<<"k = "<<k<<endl;cout<<"value = "<<ivalue<<endl;
					ivalue = ivalue<<1;
					ivalue += Char2Bit(input[k-1][p]);
				}
				_piList[p]->simulate(ivalue);
			}
		}
		simaig();
		makeFECgroup();
		//cleanFECList();
	}
	if(!b)
	{
		//sortFEC();
		cout<<ptime+input.size()<<" patterns simulated."<<endl;
	}
}

/*************************************************/
/*   Private member functions about Simulation   */
/*************************************************/
/*class CompGate
{
public:
	bool operator() (const CirGate* a, const CirGate* b)
	{ return (a->getID() < b->getID()); }
};
*/
void
CirMgr::simaig()
{
	for(size_t i=0; i<_netList.size(); ++i)
		if(_netList[i]->getTypeStr()=="AIG" || _netList[i]->getTypeStr()=="PO")
			_netList[i]->simulate();
	if(_simLog != 0)
	{
		unsigned temp;
		for(int i=0; i<32; ++i)
		{
			for(size_t k=0; k<_piList.size(); ++k)
			{
				temp = _piList[k]->getsim();
				for(int a=0; a<i; ++a)
					temp = temp>>1;
				(*_simLog) << (temp & 1);
			}
			(*_simLog) << " ";
			for(size_t q=0; q<_poList.size(); ++q)
			{
				temp = _poList[q]->getsim();
				for(int a=0; a<i; ++a)
					temp = temp>>1;
				(*_simLog) << (temp & 1);
			}
			(*_simLog) << "\n";
		}
		//(*_simLog) << "\n";
	}
}

int
CirMgr::failtimes()
{
	int pi = 2;
	int dfs = 1;
	if(_piList.size() > 2) pi = log(_piList.size()) + 1;
	if(_netList.size() > 2) dfs = log(_netList.size()) * pow(1.01, _netList.size()/1000);
	if(pi*dfs < 3) return 3;
	else if(pi*dfs > 185) return 185;
	else return pi*dfs+3;
}

void
CirMgr::makefirstgroup()
{
	GateList* firstgroup = new GateList;
	firstgroup->push_back(_inorder[0]);
	for(size_t i=0; i<_netList.size(); ++i)
	{
		if(_netList[i]->getTypeStr()=="AIG")
			firstgroup->push_back(_netList[i]);
	}
	_fecList.push_back(firstgroup);
}

bool
CirMgr::makeFECgroup()
{
	bool success = false;
	GateList* temp;
	Hash<SimHashKey, GateList* > nGrps(1);
	vector<GateList*> newGrps;
	GateList* nGroup;
	for(size_t i=0; i<_fecList.size(); ++i)
	{
		nGrps.reset();
		nGrps.init(getHashSize(_fecList[i]->size()/2));
		if(_fecList[i]->size() > 1)
		{
			for(size_t j=0; j<_fecList[i]->size(); ++j)
			{
				if(nGrps.check(SimHashKey((*_fecList[i])[j]), temp))
				{
					temp->push_back((*_fecList[i])[j]);
					//(*_fecList[i])[j]->setgrpptr((*_fecList[i])[0]->getgrpptr());
				}
				else
				{
					nGroup = new GateList;
					nGroup->push_back((*_fecList[i])[j]);
					nGrps.forceInsert(SimHashKey((*_fecList[i])[j]), nGroup);
					//for(size_t k=0; k<nGroup->size(); ++k)
					//	(*nGroup)[0]->setgrpid(newGrps.size());
					newGrps.push_back(nGroup);
					if(j != 0) success = true;
				}
			}
		}
	}
	//if(success == true)
	{
		for(size_t i=0; i<_fecList.size(); ++i)
			delete _fecList[i];
		_fecList = newGrps;
	}
	cleanFECList();
	return success;
}

void
CirMgr::cleanFECList()
{
	vector<GateList*> newGrps;
	for(size_t j=0; j<_fecList.size(); ++j)
	{
		if(_fecList[j]->size() > 1)
		{
			for(size_t k=0; k<_fecList[j]->size(); ++k)
				(*_fecList[j])[k]->setgrpptr((_fecList[j]));
			newGrps.push_back(_fecList[j]);
		}
		else
			delete _fecList[j];
	}
	//for(size_t i=0; i<_fecList.size(); ++i)
	//	delete _fecList[i];
	_fecList = newGrps;
}

/*void
CirMgr::sortFEC()
{
	for(size_t i=0; i<_fecList.size(); ++i)
		sort((*_fecList[i]).begin(), (*_fecList[i]).end(), CompGate());
}
*/
int
CirMgr::Char2Bit(char& s)
{
	if(s == '0') return 0;
	else return 1;
}
